<!DOCTYPE html>
<html>

<head>
  <meta http-equiv='content-type' value='text/html;charset=utf8'>
  <meta name='generator' value='Ronn/v0.7.3 (http://github.com/rtomayko/ronn/tree/0.7.3)'>
  <title>httpbin(1): HTTP Client Testing Service</title>
  <style type='text/css' media='all'>
    /* style: man */

    body#manpage {
      margin: 0;
    }

    .mp {
      max-width: 100ex;
      padding: 0 9ex 1ex 4ex;
    }

    .mp p,
    .mp pre,
    .mp ul,
    .mp ol,
    .mp dl {
      margin: 0 0 20px 0;
    }

    .mp h2 {
      margin: 10px 0 0 0;
    }

    .mp>p,
    .mp>pre,
    .mp>ul,
    .mp>ol,
    .mp>dl {
      margin-left: 8ex;
    }

    .mp h3 {
      margin: 0 0 0 4ex;
    }

    .mp dt {
      margin: 0;
      clear: left;
    }

    .mp dt.flush {
      float: left;
      width: 8ex;
    }

    .mp dd {
      margin: 0 0 0 9ex;
    }

    .mp h1,
    .mp h2,
    .mp h3,
    .mp h4 {
      clear: left;
    }

    .mp pre {
      margin-bottom: 20px;
    }

    .mp pre+h2,
    .mp pre+h3 {
      margin-top: 22px;
    }

    .mp h2+pre,
    .mp h3+pre {
      margin-top: 5px;
    }

    .mp img {
      display: block;
      margin: auto;
    }

    .mp h1.man-title {
      display: none;
    }

    .mp,
    .mp code,
    .mp pre,
    .mp tt,
    .mp kbd,
    .mp samp,
    .mp h3,
    .mp h4 {
      font-family: monospace;
      font-size: 14px;
      line-height: 1.42857142857143;
    }

    .mp h2 {
      font-size: 16px;
      line-height: 1.25;
    }

    .mp h1 {
      font-size: 20px;
      line-height: 2;
    }

    .mp {
      text-align: justify;
      background: #fff;
    }

    .mp,
    .mp code,
    .mp pre,
    .mp pre code,
    .mp tt,
    .mp kbd,
    .mp samp {
      color: #131211;
    }

    .mp h1,
    .mp h2,
    .mp h3,
    .mp h4 {
      color: #030201;
    }

    .mp u {
      text-decoration: underline;
    }

    .mp code,
    .mp strong,
    .mp b {
      font-weight: bold;
      color: #131211;
    }

    .mp em,
    .mp var {
      font-style: italic;
      color: #232221;
      text-decoration: none;
    }

    .mp a,
    .mp a:link,
    .mp a:hover,
    .mp a code,
    .mp a pre,
    .mp a tt,
    .mp a kbd,
    .mp a samp {
      color: #0000ff;
    }

    .mp b.man-ref {
      font-weight: normal;
      color: #434241;
    }

    .mp pre {
      padding: 0 4ex;
    }

    .mp pre code {
      font-weight: normal;
      color: #434241;
    }

    .mp h2+pre,
    h3+pre {
      padding-left: 0;
    }

    ol.man-decor,
    ol.man-decor li {
      margin: 3px 0 10px 0;
      padding: 0;
      float: left;
      width: 33%;
      list-style-type: none;
      text-transform: uppercase;
      color: #999;
      letter-spacing: 1px;
    }

    ol.man-decor {
      width: 100%;
    }

    ol.man-decor li.tl {
      text-align: left;
    }

    ol.man-decor li.tc {
      text-align: center;
      letter-spacing: 4px;
    }

    ol.man-decor li.tr {
      text-align: right;
      float: right;
    }
  </style>
  <style type='text/css' media='all'>
    /* style: 80c */

    .mp {
      max-width: 86ex
    }

    ul {
      list-style: None;
      margin-left: 1em !important
    }

    .man-navigation {
      left: 101ex
    }

    .scheme-container {
      display: none !important;
    }
  </style>
</head>

<body id='manpage'>
  <a href="https://github.com/psf/httpbin" class="github-corner" aria-label="View source on Github">
    <svg width="80" height="80" viewBox="0 0 250 250" style="fill:#151513; color:#fff; position: absolute; top: 0; border: 0; right: 0;"
      aria-hidden="true">
      <path d="M0,0 L115,115 L130,115 L142,142 L250,250 L250,0 Z"></path>
      <path d="M128.3,109.0 C113.8,99.7 119.0,89.6 119.0,89.6 C122.0,82.7 120.5,78.6 120.5,78.6 C119.2,72.0 123.4,76.3 123.4,76.3 C127.3,80.9 125.5,87.3 125.5,87.3 C122.9,97.6 130.6,101.9 134.4,103.2"
        fill="currentColor" style="transform-origin: 130px 106px;" class="octo-arm"></path>
      <path d="M115.0,115.0 C114.9,115.1 118.7,116.5 119.8,115.4 L133.7,101.6 C136.9,99.2 139.9,98.4 142.2,98.6 C133.8,88.0 127.5,74.4 143.8,58.0 C148.5,53.4 154.0,51.2 159.7,51.0 C160.3,49.4 163.2,43.6 171.4,40.1 C171.4,40.1 176.1,42.5 178.8,56.2 C183.1,58.6 187.2,61.8 190.9,65.4 C194.5,69.0 197.7,73.2 200.1,77.6 C213.8,80.2 216.3,84.9 216.3,84.9 C212.7,93.1 206.9,96.0 205.4,96.6 C205.1,102.4 203.0,107.8 198.3,112.5 C181.9,128.9 168.3,122.5 157.7,114.1 C157.9,116.9 156.7,120.9 152.7,124.9 L141.0,136.5 C139.8,137.7 141.6,141.9 141.8,141.8 Z"
        fill="currentColor" class="octo-body"></path>
    </svg>
  </a>



  {% include 'httpbin.1.html' %} {% if tracking_enabled %} {% include 'trackingscripts.html' %} {% endif %}

</body>

</html>
